"""Tests for mdpo text utilities."""

import pytest

from mdpo.text import (
    max_char_in_a_row,
    min_not_max_chars_in_a_row,
)


@pytest.mark.parametrize(("char", "text", "expected_result"), (
    ("`", "hello `` ``` a `` `", 3),
    ("`", "hello `` ``` a ```` `", 4),
    ("`", "hello `` ``` a `` `````", 5),

    # does not work for multiple characters
    ("``", "hello `` ``` a `` `````", 0)
))
def test_max_char_in_a_row(char, text, expected_result):
    assert max_char_in_a_row(char, text) == expected_result


@pytest.mark.parametrize(("char", "text", "expected_result"), (
    ("`", "hello `` ``` a `` `", 4),
    ("`", "hello `` ``` a ```` `", 5),
    ("`", "hello `` ``` a `` `````", 1),
    ("`", "hello ` `` ``` a `` `````", 4),

    # as default, 1
    ("`", "hello", 1),

    # does not work for multiple characters (returns default)
    ("``", "hello `` ``` a `` `````", 1)
))
def test_min_not_max_chars_in_a_row(char, text, expected_result):
    assert min_not_max_chars_in_a_row(char, text) == expected_result



"""HTML utilities for mdpo."""


def html_attrs_tuple_to_string(attrs):
    """Converts a set of HTML attributes tuple to an HTML string."""
    response = ''
    for i, (name, value) in enumerate(attrs):
        response += '%s' % name
        if value is not None:
            response += '="%s"' % value
        if i < len(attrs) - 1:
            response += ' '
    return response


def get_html_attrs_tuple_attr(attrs, attrname):
    """Returns the value of an attribute from an attributes tuple given an
    attribute name."""
    response = None
    for name, value in attrs:
        if name == attrname:
            response = value
            break
    return response



"""mdpo commands related utilities."""

import re

COMMAND_SEARCH_RE = re.compile(
    r'<\!\-\-\s{0,1}mdpo\-([a-z\-]+)\s{0,1}([\w\s]+)?\-\->')


def search_html_command(text):
    """Search a mdpo HTML command inside text."""
    command_search = COMMAND_SEARCH_RE.search(text)
    if command_search:
        return command_search.groups()
    return (None, None)



#!/usr/bin/env python

"""md2po command line interface."""

import argparse
import sys
try:
    from itertools import izip
except ImportError:
    izip = zip

from mdpo import __version__
from mdpo.cli import parse_list_argument
from mdpo.md2po import markdown_to_pofile
from mdpo.md4c import DEFAULT_MD4C_FLAGS


DESCRIPTION = ('Utility like xgettext to extract Markdown content dumping it'
               ' inside pofiles.')


def build_parser():
    parser = argparse.ArgumentParser(description=DESCRIPTION)
    parser.add_argument('-v', '--version', action='version',
                        version='%(prog)s ' + __version__,
                        help='Show program version number and exit.')
    parser.add_argument('-q', '--quiet', action='store_true',
                        help='Do not print output to STDOUT.')
    parser.add_argument('glob_or_content', metavar='GLOB_OR_CONTENT',
                        help='Glob to markdown input files or markdown'
                             ' content as a string. If not provided,'
                             ' will be read from STDIN.')
    parser.add_argument('-i', '--ignore', dest='ignore', default=[],
                        help='List of filepaths to ignore if'
                             ' ``GLOB_OR_CONTENT`` argument is a glob,'
                             ' as a list of comma separated values.',
                        metavar='PATH_1,PATH_2...')
    parser.add_argument('-po', '--po-filepath', dest='po_filepath',
                        default=None,
                        help='Merge new msgids in the po file indicated'
                             ' at this parameter (if ``--save`` argument'
                             ' is passed) or use the msgids of the file'
                             ' as reference for'
                             ' ``--mark-not-found-as-obsolete`` parameter.',
                        metavar='OUTPUT_PO_FILEPATH')
    parser.add_argument('-s', '--save', dest='save', action='store_true',
                        help='Save new found msgids to the po file'
                             ' indicated as parameter ``--filepath``.')
    parser.add_argument('-mo', '--mo-filepath', dest='mo_filepath',
                        default=None,
                        help='The resulting pofile will be compiled to a'
                             ' mofile and saved in the path specified at'
                             ' this parameter.',
                        metavar='OUTPUT_MO_FILEPATH')
    parser.add_argument('-p', '--plaintext', dest='plaintext',
                        action='store_true',
                        help='Do not include markdown markup characters in'
                             ' extracted msgids for **bold text**, *italic'
                             ' text*, `inline code` and [link](target).')
    parser.add_argument('-w', '--wrapwidth', dest='wrapwidth',
                        help='Wrap width for po file indicated at'
                             ' ``--filepath`` parameter. Only useful when'
                             ' the ``-w`` option was passed to xgettext.',
                        metavar='N', type=int)
    parser.add_argument('-m', '--merge-pofiles',
                        dest='mark_not_found_as_absolete',
                        action='store_false',
                        help='New found msgids not present in the pofile'
                             ' passed at ``--po-filepath`` parameter will be'
                             ' preserved in the new pofile, even when are not'
                             ' been found in the current extraction.'
                             ' If this argument is not passed, will be marked'
                             ' as obsolete strings.')
    parser.add_argument('-f', '--flags',
                        default=DEFAULT_MD4C_FLAGS, dest='flags',
                        help='md4c extensions used to parse markdown'
                             ' content, separated by ``|`` or ``+``'
                             '  characters. You can see all available at http'
                             's://github.com/mity/md4c#markdown-extensions')
    parser.add_argument('-e', '--encoding', dest='encoding', default=None,
                        help='Resulting pofile encoding (autodetected by'
                             ' default).')
    parser.add_argument('-x', '--xheaders', dest='xheaders',
                        action='store_true',
                        help='Include mdpo specification x-headers.'
                             ' These only will be included if you do not pass'
                             ' the parameter ``--plaintext``.')
    return parser


def parse_options(args=[]):
    parser = build_parser()
    if '-h' in args or '--help' in args:
        parser.print_help()
        sys.exit(0)
    opts, unknown = parser.parse_known_args(args)

    if not sys.stdin.isatty():
        opts.glob_or_content = sys.stdin.read().strip('\n')
    if opts.ignore:
        opts.ignore = parse_list_argument(opts.ignore)

    return opts


def run(args=[]):
    opts = parse_options(args)

    kwargs = dict(
        po_filepath=opts.po_filepath,
        ignore=opts.ignore,
        save=opts.save,
        mo_filepath=opts.mo_filepath,
        plaintext=opts.plaintext,
        mark_not_found_as_absolete=opts.mark_not_found_as_absolete,
        flags=opts.flags,
        encoding=opts.encoding,
        xheaders=opts.xheaders)
    if isinstance(opts.wrapwidth, int):
        kwargs['wrapwidth'] = opts.wrapwidth

    pofile = markdown_to_pofile(opts.glob_or_content, **kwargs)

    if not opts.quiet:
        sys.stdout.write('%s\n' % pofile.__unicode__())

    return (pofile, 0)


def main():
    sys.exit(run(args=sys.argv[1:])[1])


if __name__ == '__main__':
    main()



import pytest

from mdpo.md4c import (
    DEFAULT_MD4C_FLAGS,
    parse_md4c_flags_string,
)

DEFAULT_MODES_OUTPUT = {
    'strikethrough': True,
    'latexmathspans': True,
    'wikilinks': True,
    'underline': False,
}


@pytest.mark.parametrize(('flags_string', 'flags_sum', 'modes'), (
    (
        DEFAULT_MD4C_FLAGS,
        15105,
        DEFAULT_MODES_OUTPUT,
    ),
    (
        DEFAULT_MD4C_FLAGS + '|FOOBARUNEXISTENTFLAGNOTPOSSIBLE',
        15105,
        DEFAULT_MODES_OUTPUT
    ),
    (
        ('MD_FLAG_COLLAPSEWHITESPACE|'
         'MD_FLAG_TABLES|'
         'MD_FLAG_TASKLISTS|'
         'MD_FLAG_WIKILINKS'),
        10497,
        {
            'strikethrough': False,
            'latexmathspans': False,
            'wikilinks': True,
            'underline': False,
        }
    )
))
def test_parse_md4c_flags_string(flags_string, flags_sum, modes):
    flags_sum_result, modes_result = parse_md4c_flags_string(flags_string)
    assert flags_sum_result == flags_sum
    assert modes_result == modes



"""mdpo package"""

from mdpo.md2po import markdown_to_pofile
from mdpo.po2md import pofile_to_markdown
from mdpo.mdpo2html import markdown_pofile_to_html


__version__ = '0.2.17'
__version_info__ = tuple([int(i) for i in __version__.split('.')])
__title__ = 'mdpo'
__description__ = ('Utilities to translate Markdown files using `.po`'
                   ' files.')
__all__ = ("markdown_to_pofile",
           "pofile_to_markdown",
           "markdown_pofile_to_html")



"""Markdown related utilities for mdpo."""

import re


def escape_links_titles(text, link_start_string='[', link_end_string=']'):
    """Finds titles inside inline links or images and escapes `"` characters
    found inside, returning the replaced string.
    """
    link_end_string_escaped_regex = re.escape(link_end_string)
    regex = r'(%s[^%s]+%s\([^\s]+\s)([^\)]+)' % (
        re.escape(link_start_string),
        link_end_string_escaped_regex,
        link_end_string_escaped_regex,
    )

    for match in re.findall(regex, text):
        original_string = match[0] + match[1]
        target_string = match[0] + '"%s"' % (
            match[1][1:-1].replace('"', '\\"'))
        text = text.replace(original_string, target_string)
    return text


def inline_untexted_links(text, link_start_string='[', link_end_string=']'):
    """Given a string like ``"String with [self-referenced-link]"``, replaces
    self referenced links markup characters by new ones, in this case
    ``"String with <self-referenced-link>"``. Only replaces links that do not
    contains hrefs.
    Wikilinks are not replaced (strings started with ``[[`` and ended with
    ``]]`` string chunks).
    """
    link_end_string_escaped_regex = re.escape(link_end_string)
    regex = r'(%s)([^%s]+)(%s)(?!\[|\()(?!\])' % (
        re.escape(link_start_string),
        link_end_string_escaped_regex,
        link_end_string_escaped_regex,
    )
    return re.sub(regex, r'<\g<2>>', text)


def n_chars_until_chars(text, chars=[" ", "\n"]):
    """Returns the number of characters until one of the characters passed
    as ``chars`` argument is found.
    """
    n = 1
    for ch in text:
        if ch in chars:
            break
        n += 1
    return n


def fixwrap_codespans(lines, code_start_string='`', code_end_string='`',
                      width=80):
    """Given a set of lines wrapped by `:py:class:textwrap.TextWrapper`,
    unwraps reasonably all markdown codespans that are found inside the lines.
    This funcion is designed to render codespans without wrap them in multiples
    lines, which is desirable in markdown rendering based on available linters.
    """
    def _chars_num_until_next_codespan_exit(text):
        __exiting, __inside, __entering = (False, False, False)
        __n_backticks_wrapper, __n_backticks_to_exit = (0, None)
        n = 1
        _prev_char = None
        for _ch in text:
            if not __inside:
                if not __entering and not __exiting and \
                        _ch == code_start_string:
                    if _prev_char != "\\":
                        __entering = True
                        __n_backticks_wrapper += 1
                elif __entering and _ch != code_start_string:
                    __inside = True
                    __entering = False
                    __n_backticks_to_exit = __n_backticks_wrapper
                elif __entering and _ch == code_start_string:
                    if _prev_char != "\\":
                        __n_backticks_wrapper += 1
            elif __inside:
                if not __exiting and not __entering and \
                        _ch == code_end_string:
                    if _prev_char != "\\":
                        __exiting = True
                        __n_backticks_to_exit -= 1
                elif __exiting and _ch != code_end_string:
                    if __n_backticks_to_exit == 0:
                        break
                    else:
                        __exiting = False
                        __n_backticks_to_exit = __n_backticks_wrapper
                elif __exiting and _ch == code_end_string:
                    if _prev_char != "\\":
                        __n_backticks_to_exit -= 1
            _prev_char = _ch
            n += 1

        return n

    response = []

    _curr_line = ''

    _entering_codespan, _exiting_codespan, _inside_codespan = (
        False, False, False)
    _codespan_n_backticks_wrapper, _n_backticks_to_exit_codespan = (0, None)
    prev_char = None
    for li, line in enumerate(lines):
        for ci, ch in enumerate(line):
            if not _inside_codespan:
                if not _entering_codespan and not _exiting_codespan and \
                        ch == code_start_string:

                    # check '`' escapes
                    if prev_char != "\\":
                        _entering_codespan = True
                        _codespan_n_backticks_wrapper += 1

                    # 70% of the line must be filled to wrap before codespans
                    if len(_curr_line) > width * .7:
                        _entering_codespan_length = \
                            _chars_num_until_next_codespan_exit(
                                line[ci:] + ' '.join(lines[li+1:]))
                        if len(_curr_line) + _entering_codespan_length > width:
                            response.append(_curr_line.rstrip(" "))
                            _curr_line = ''
                elif _entering_codespan and ch != code_start_string:
                    _inside_codespan = True
                    _entering_codespan = False
                    _n_backticks_to_exit_codespan = \
                        _codespan_n_backticks_wrapper
                elif _entering_codespan and ch == code_start_string:
                    if prev_char != "\\":
                        _codespan_n_backticks_wrapper += 1
            elif _inside_codespan:
                if not _exiting_codespan and not _entering_codespan and \
                        ch == code_end_string:
                    if prev_char != "\\":
                        _exiting_codespan = True
                        _n_backticks_to_exit_codespan -= 1
                elif _exiting_codespan and ch != code_end_string:
                    if _n_backticks_to_exit_codespan == 0:
                        _exiting_codespan = False
                        _inside_codespan = False
                        _codespan_n_backticks_wrapper = 0
                    else:
                        _exiting_codespan = False
                        _n_backticks_to_exit_codespan = \
                            _codespan_n_backticks_wrapper
                elif _exiting_codespan and ch == code_end_string:
                    if prev_char != "\\":
                        _n_backticks_to_exit_codespan -= 1

            if not _inside_codespan:
                if ch in [" ", "\n"]:
                    # check if we can add more words until next space
                    _chars_until_next_space = n_chars_until_chars(
                        line[ci+1:] + ' ' + ' '.join(lines[li+1:]))
                    if len(_curr_line) + _chars_until_next_space > width:
                        response.append(_curr_line.rstrip(" "))
                        _curr_line = ''

            # store a reference to previous character
            prev_char = ch

            if ch == " " and not _curr_line:
                continue
            _curr_line += ch

        if not _inside_codespan:
            _chars_until_next_space = n_chars_until_chars(
                ' '.join(lines[li+1:]))
            if len(_curr_line) + _chars_until_next_space > width:
                response.append(_curr_line.rstrip(" "))
                _curr_line = ''
            elif _curr_line:
                _curr_line += ' '
        elif _curr_line:
            _curr_line += ' '

    if _curr_line:
        response.append(_curr_line.rstrip(" "))
    return response



"""HTML produced from Markdown files translator using pofiles as reference."""

import glob
import re
from collections import OrderedDict
import html
from html.parser import HTMLParser

import polib
import md4c

from mdpo.html import (
    get_html_attrs_tuple_attr,
    html_attrs_tuple_to_string,
)
from mdpo.io import (
    filter_paths,
    to_file_content_if_is_file,
)

PROCESS_REPLACER_TAGS = ['p', 'li', 'h1', 'h2', 'h3',
                         'h4', 'h5', 'h6', 'td', 'th']
ALIGNMENT_CHARS = ['\n', ' ', '\t', '\r']


class MdPo2HTML(HTMLParser):
    def __init__(self, pofiles, ignore=[], merge_adjacent_markups=False,
                 code_tags=['code'], bold_tags=['b', 'strong'],
                 italic_tags=['em', 'i'], link_tags=['a'],
                 ignore_grouper_tags=['div', 'hr']):
        self.pofiles = [polib.pofile(pofilepath) for pofilepath in
                        filter_paths(glob.glob(pofiles), ignore_paths=ignore)]
        self.output = ''
        self.replacer = []
        self._raw_replacement = ''
        self.context = []

        # lazy translators mode
        self.merge_adjacent_markups = merge_adjacent_markups

        # code markup
        self.code_tags = code_tags

        # bold markup
        self.bold_tags = bold_tags

        # italic markup
        self.italic_tags = italic_tags

        # link markup
        self.link_tags = link_tags

        self.ignore_grouper_tags = ignore_grouper_tags

        self.markups_handler = OrderedDict(sorted(
            {
                'bold': {
                    'start_string': '**',
                    'end_string': '**',
                    'tags': self.bold_tags,
                },
                'italic': {
                    'start_string': '*',
                    'end_string': '*',
                    'tags': self.italic_tags,
                },
            }.items(),
            key=lambda k: -max([len(k[1]['start_string']),
                                len(k[1]['end_string'])]),
        ))

        self.markup_tags = []
        self.markup_tags.extend(self.code_tags)
        self.markup_tags.extend(self.bold_tags)
        self.markup_tags.extend(self.italic_tags)

        self.html_renderer = md4c.HTMLRenderer(md4c.MD_FLAG_TABLES)

        super().__init__()

    def _merge_adyacent_tags(self, html, template_tags):
        for markup, mk_data in self.markups_handler.items():
            regexes = []
            for tag in mk_data['tags']:
                if tag not in template_tags:
                    continue
                for _tag in mk_data['tags']:
                    if _tag not in template_tags:
                        continue
                    regex = r'</%s>\s*<%s>' % (tag, _tag)
                    if regex not in regexes:
                        regexes.append(regex)

            for regex in regexes:
                html = re.sub(regex, ' ', html)
        return html

    def _process_replacer(self):
        # print("REPLACER:", self.replacer)

        template_tags = []
        raw_html_template, _current_replacement = ('', '')

        _current_link_target = ''

        _last_start_tag = None
        _last_end_tag = None
        _inside_code = False
        while self.replacer:
            handle, handled, attrs = self.replacer.pop(0)
            if handle == 'start':
                template_tags.append(handled)

                if handled in self.code_tags:
                    _current_replacement += '`'
                    raw_html_template += '<%s%s>' % (
                        handled,
                        (' ' + html_attrs_tuple_to_string(attrs)
                         if attrs else '')
                    )
                elif handled in self.bold_tags:
                    _current_replacement += '**'
                    raw_html_template += '<%s%s>' % (
                        handled,
                        (' ' + html_attrs_tuple_to_string(attrs)
                         if attrs else '')
                    )
                elif handled in self.italic_tags:
                    _current_replacement += '*'
                    raw_html_template += '<%s%s>' % (
                        handled,
                        (' ' + html_attrs_tuple_to_string(attrs)
                         if attrs else '')
                    )
                elif handled in self.link_tags:
                    title = get_html_attrs_tuple_attr(attrs, "title")
                    _current_link_target += '(%s' % get_html_attrs_tuple_attr(
                        attrs, "href")
                    if title:
                        _current_link_target += ' "%s"' % title
                    _current_link_target += ')'

                    raw_html_template += '<%s' % handled

                    attrs_except_href_title = []
                    for attr, value in attrs:
                        if attr in ['title', 'href']:
                            raw_html_template += ' %s="{}"' % attr
                        else:
                            attrs_except_href_title.append(attr)
                    raw_html_template += html_attrs_tuple_to_string(
                        attrs_except_href_title) + '>'
                else:
                    raw_html_template += '<%s%s>' % (
                        handled,
                        ' ' + html_attrs_tuple_to_string(attrs)
                        if attrs else '')
                _last_start_tag = handled
                if _last_start_tag == 'code':
                    _inside_code = True

            elif handle == 'data':
                if not _inside_code:
                    handled = re.sub(r'\s{2,}', ' ', handled)

                if all((ch in ALIGNMENT_CHARS) for ch in handled):
                    raw_html_template += handled
                    if _last_start_tag in self.markup_tags:
                        _current_replacement += handled
                    else:
                        _current_replacement += handled
                else:
                    raw_html_template += '{}'
                    if _current_link_target:
                        _current_replacement += '[%s]%s' % (
                            handled, _current_link_target)
                        _current_link_target = ''
                    else:
                        _current_replacement += handled
            elif handle == 'end':
                raw_html_template += '</%s>' % handled
                if handled in self.code_tags:
                    _current_replacement += '`'
                elif handled in self.bold_tags:
                    _current_replacement += '**'
                elif handled in self.italic_tags:
                    _current_replacement += '*'
                _last_end_tag = handled
                if _last_end_tag == 'code':
                    _inside_code = False
            elif handle == 'comment':
                raw_html_template += '<!--%s-->' % handled
            elif handle == 'startend':
                raw_html_template += '<%s%s/>' % (
                    handled,
                    (' ' + html_attrs_tuple_to_string(attrs) if attrs else '')
                )

        _current_replacement = html.unescape(_current_replacement)

        replacement = self.translations.get(
            _current_replacement, _current_replacement)
        if not replacement:
            replacement = _current_replacement

        for tt in template_tags:
            for tags_group in [self.bold_tags, self.italic_tags]:
                if tt in tags_group:
                    for tag in tags_group:
                        if tag not in template_tags:
                            template_tags.append(tag)

        # print("RAW TEMPLATE:", raw_html_template)
        # print("TEMPLATE TAGS:", template_tags)
        # print('CURRENT MSGID: \'%s\'' % _current_replacement)
        # print('MSGSTR:', replacement)

        html_before_first_replacement = raw_html_template.split('{')[0]
        html_after_last_replacement = raw_html_template.split('}')[-1]
        for tags_group in [self.bold_tags, self.italic_tags, self.code_tags]:
            for tag in tags_group:
                html_before_first_replacement = \
                    html_before_first_replacement.split('<%s>' % tag)[0]
                html_after_last_replacement = \
                    html_after_last_replacement.split('</%s>' % tag)[-1]
            html_before_first_replacement = \
                html_before_first_replacement.split('<a href="')[0]

        html_inner = '\n'.join(
            self.html_renderer.parse(replacement).split('\n')[:-1])
        html_inner = '>'.join(html_inner.split('>')[1:])
        html_inner = '<'.join(html_inner.split('<')[:-1])

        html_template = html_before_first_replacement + html_inner + \
            html_after_last_replacement

        if self.merge_adjacent_markups:
            html_template = self._merge_adyacent_tags(html_template,
                                                      template_tags)

        self.output += html_template
        self.context = []

        # print("________________________________________________")

    def handle_starttag(self, tag, attrs):
        # print("START TAG: %s | POS: %d:%d" % (tag, *self.getpos()))

        if tag in self.ignore_grouper_tags:
            self.context.append(tag)
            self.output += '<%s%s>' % (
                tag, ' ' + html_attrs_tuple_to_string(attrs) if attrs else '')
        elif self.context and self.context[0] in self.ignore_grouper_tags:
            self.output += '<%s%s>' % (
                tag, ' ' + html_attrs_tuple_to_string(attrs) if attrs else '')
        elif tag == 'ul' and not self.context:
            self.output += '<%s%s>' % (
                tag, ' ' + html_attrs_tuple_to_string(attrs) if attrs else '')
        elif tag in ['blockquote', 'table', 'thead', 'tbody', 'tr']:
            self.output += '<%s%s>' % (
                tag, ' ' + html_attrs_tuple_to_string(attrs) if attrs else '')
        else:
            self.replacer.append(('start', tag, attrs))
            self.context.append(tag)

    def handle_endtag(self, tag):
        # print("END TAG: %s | POS: %d:%d" % (tag, *self.getpos()))

        if tag in self.ignore_grouper_tags:
            self.output += '</%s>' % tag
            if self.context:
                self.context.pop()
        elif self.context and self.context[0] in self.ignore_grouper_tags:
            self.output += '</%s>' % tag
        elif tag in PROCESS_REPLACER_TAGS:
            self.replacer.append(('end', tag, None))
            self._process_replacer()
        elif tag in ['ul', 'blockquote', 'tr', 'table', 'thead', 'tbody']:
            self.output += '</%s>' % tag
        else:
            self.replacer.append(('end', tag, None))
            if self.context:
                self.context.pop()

    def handle_startendtag(self, tag, attrs):
        # print("STARTEND TAG: %s | POS: %d:%d" % (tag, *self.getpos()))

        if not self.replacer:
            self.output += self.get_starttag_text()
        else:
            self.replacer.append(('startend', tag, attrs))

    def handle_data(self, data):
        # print("     DATA: '%s'" % (data))

        if data:
            if not self.replacer or (
                    self.context and
                    self.context[0] in self.ignore_grouper_tags) or \
                    not self.context:
                self.output += data
                return
            else:
                if self.context:
                    data = data.replace('\n', ' ')
                self.replacer.append(('data', data, None))

    def handle_comment(self, data):
        # print("     COMMENT: '%s'" % (data))

        if not self.replacer:
            self.output += '<!--%s-->' % data
        else:
            self.replacer.append(('comment', data, None))

    def translate(self, filepath_or_content, save=None):
        content = to_file_content_if_is_file(filepath_or_content)

        self.translations = {}
        for pofile in self.pofiles:
            for entry in pofile:
                self.translations[entry.msgid] = entry.msgstr

        self.feed(content)

        if save:
            with open(save, "w") as f:
                f.write(self.output)

        self.reset()

        return self.output


def markdown_pofile_to_html(filepath_or_content, pofiles, ignore=[],
                            save=None, **kwargs):
    return MdPo2HTML(
        pofiles, ignore=ignore, **kwargs
    ).translate(filepath_or_content, save=save)



"""Text utilities for mdpo."""


def max_char_in_a_row(char, text):
    """Returns the maximum numbers of characters in a row found inside a string.

    Args:
        char (str): Character to search.
        text (str): Text inside which find the character repeated in a row.

    Returns:
        int: Maximum repeats in a row of the character inside the text.
    """
    response, partial_response = (0, 0)
    _in_the_row = False

    for ch in text:
        if ch == char:
            partial_response += 1
            _in_the_row = True
        elif _in_the_row:
            _in_the_row = False
            response = max(response, partial_response)
            partial_response = 0
    return max(response, partial_response)


def min_not_max_chars_in_a_row(char, text, default=1):
    """Returns the minimum possible of characters not found in a row for a string.
    For example, given the string ``"c cc cccc"`` and the character ``"c"``,
    returns the number of characters in a row that are not found, so ``3`` in
    this case.
    This function is used in the source code to compute the string that
    wraps markdown code spans. Given the code span
    ``"code that contains 3 ``` and 2 `` backticks"`` and the character
    ``"`"``, this function returns  ``1``.

    Args:
        char (str): Character to search.
        text (str): Text inside which find the character repeated in a row.

    Returns:
        int: Minimum number possible of characters not found in a row.
    """
    in_a_rows, _current_in_a_row, _in_the_row = ([], 0, False)
    for ch in text:
        if ch == char:
            _current_in_a_row += 1
            _in_the_row = True
        elif _in_the_row:
            _in_the_row = False
            if _current_in_a_row not in in_a_rows:
                in_a_rows.append(_current_in_a_row)
            _current_in_a_row = 0
    if _in_the_row and _current_in_a_row not in in_a_rows:
        in_a_rows.append(_current_in_a_row)

    if in_a_rows:
        response = None
        for n in range(1, max(in_a_rows)+2):
            if n not in in_a_rows:
                response = n
                break
    else:
        response = default
    return response


def striplastline(text):
    return '\n'.join(text.split('\n')[:-1])



"""md4c related stuff for mdpo"""

import md4c


DEFAULT_MD4C_FLAGS = ('MD_FLAG_COLLAPSEWHITESPACE|'
                      'MD_FLAG_TABLES|'
                      'MD_FLAG_STRIKETHROUGH|'
                      'MD_FLAG_TASKLISTS|'
                      'MD_FLAG_LATEXMATHSPANS|'
                      'MD_FLAG_WIKILINKS')


def parse_md4c_flags_string(flags_string):
    modes = {
        "strikethrough": False,
        "latexmathspans": False,
        "wikilinks": False,
        "underline": False,  # unactive by default
    }
    flags_string = flags_string.replace('+', '|').replace(' ', '')
    flags_list = []
    for flag in flags_string.split('|'):
        if not hasattr(md4c, flag):
            continue
        md4c_attr = getattr(md4c, flag)
        flags_list.append(md4c_attr)
        if md4c_attr == md4c.MD_FLAG_STRIKETHROUGH:
            modes["strikethrough"] = True
        elif md4c_attr == md4c.MD_FLAG_LATEXMATHSPANS:
            modes["latexmathspans"] = True
        elif md4c_attr == md4c.MD_FLAG_WIKILINKS:
            modes['wikilinks'] = True
        elif md4c_attr == md4c.MD_FLAG_UNDERLINE:
            modes['underline'] = True
    return (sum(flags_list), modes)
